-------------------------------------------------------
--! @file decoder_2to4en_tb.vhd
--! @brief Testbench for 2 to 4 decoder with enable
--! @author Edson S. Gomi (gomi@usp.br)
--! @date 2020-03-29
-------------------------------------------------------

--  A testbench has no ports.
entity decoder_2to4en_tb is
end entity decoder_2to4en_tb;

architecture testbench of decoder_2to4en_tb is

  --  Declaration of the component to be tested.  
  component decoder_2to4en
  port (
    EN : in bit;
    A : in bit_vector (1 downto 0);
    Y_L : out bit_vector (3 downto 0)
    );
  end component decoder_2to4en;

  -- Declaration of signals
  signal en : bit;
  signal a : bit_vector (1 downto 0);
  signal y_l : bit_vector (3 downto 0);

begin
  -- Component instantiation
  -- DUT = Device Under Test 
  DUT: entity work.decoder_2to4en(dataflow) port map (
    EN => en,
    A => a,
    Y_L => y_l
    );

  --  This process does the real job.
  stimulus_process: process is
    type pattern_type is record
      --  The inputs of the circuit.
      en : bit;
      a : bit_vector (1 downto 0);
      
      --  The expected outputs of the circuit.
      y_l : bit_vector (3 downto 0);
    end record;

    --  The patterns to apply.
    type pattern_array is array (natural range <>) of pattern_type;
    constant patterns : pattern_array :=
      (
        ('0',"00","1111"),
        ('0',"01","1111"),
        ('0',"10","1111"),
        ('0',"11","1111"),
        ('1',"00","1110"),
        ('1',"01","1101"),
        ('1',"10","1011"),
        ('1',"11","0111"));
        
  begin        
    --  Check each pattern.
    for k in patterns'range loop

      --  Set the inputs.
      en <= patterns(k).en;
      a <= patterns(k).a;
      
       --  Wait for the results.
      wait for 1 ns;
      
  --  Check the outputs.

      assert y_l = patterns(k).y_l        
        report "bad y" severity error;
    end loop;
    
    assert false report "end of test" severity note;
    
  --  Wait forever; this will finish the simulation.
    wait;
  end process;


  
end architecture testbench;
  

