-------------------------------------------------------
--! @file batalha_nava_tb.vhd
--! @brief Testbench for Batalha Naval
--! @author Edson S. Gomi (gomi@usp.br)
--! @date 2020-04-01
-------------------------------------------------------

entity batalha_naval_tb is
end entity batalha_naval_tb;

architecture testbench of batalha_naval_tb is

--  Declaration of the component that will be instantiated.  
  component batalha_naval    
    port (
      T   : in bit_vector(15 downto 0);   -- Player 1
      C,L : in bit_vector(3 downto 0);    -- Player 2
      S   : out bit   -- Speaker "chuá" (erro) or "bum" (acerto)
      );
  end component batalha_naval;

  signal t : bit_vector(15 downto 0);
  signal c,l : bit_vector(3 downto 0);
  signal s : bit;
  
begin
  --  Component instantiation.
  DUT: entity work.batalha_naval port map (
    T => t,
    C => c,
    L => l,
    S => s
    );

  --  This process does the real job.
  stimulus_process: process is
    type pattern_type is record
      --  The inputs of the circuit.
      t : bit_vector (15 downto 0);
      c : bit_vector (3 downto 0);
      l : bit_vector (3 downto 0);
      
      --  The expected outputs of the circuit.
      s : bit;
    end record;

    --  The patterns to apply.
    type pattern_array is array (natural range <>) of pattern_type;
    constant patterns : pattern_array :=
      (
        ("1000000000000000","1000","1000",'1'),
        ("1000000000000000","1000","0100",'0'),
        ("0000010000000000","0100","0100",'1'),
        ("0000010000000000","1000","0100",'0'));

  begin        
    --  Check each pattern.
    for k in patterns'range loop

      --  Set the inputs.
      t <= patterns(k).t;
      c <= patterns(k).c;
      l <= patterns(k).l;
      
       --  Wait for the results.
      wait for 1 ns;
      
  --  Check the outputs.

      assert s = patterns(k).s        
        report "bad result" severity error;

    end loop;
    
    assert false report "end of test" severity note;
    
  --  Wait forever; this will finish the simulation.
    wait;
  end process;
  
end architecture testbench;
  

